package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;
import java.awt.event.KeyAdapter;
import org.jdesktop.lg3d.wonderland.appshare.AppWindowGraphics2DApp;
import org.jdesktop.lg3d.wonderland.appshare.SimpleControlArb;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.event.MouseMotionListener;
import org.jdesktop.lg3d.wonderland.appshare.AppGroup;
import org.jdesktop.lg3d.wonderland.darkstar.client.cell.SharedApp2DImageCell;
import javax.swing.JPanel;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.ITask;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.TaskManager;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.IProject;

import org.jdesktop.lg3d.wonderland.darkstar.client.ChannelController;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.Message;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DCellChangeMessage;


/**
 * UCViewer2DApp.java
 * 
 * The application for the use case viewer cell
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/5/08
 *
 */
public class UCViewer2DApp extends AppWindowGraphics2DApp {

	/**
	 * the sizes
	 */
    private static final int DEFAULT_WIDTH = 1500;
    private static final int DEFAULT_HEIGHT = 800;
    private int preferredWidth = DEFAULT_WIDTH;
    private int preferredHeight = DEFAULT_HEIGHT;
    
    /**
     * the current panel
     */
    protected JPanel curPanel;
    
    /**
     * the task manager
     */
    protected TaskManager taskManager;
    
    /**
     * the current project 
     */
    protected IProject currentProject = null; // null by default
    
    /**
     * the current task
     */
    protected ITask currentTask = null; // null by default
    
    /**
     * creates a default veiwer
     * @param cell
     */
    public UCViewer2DApp(SharedApp2DImageCell cell) {
        this(cell, 0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT);
        
        // generate task manager
        taskManager = TaskManager.getInstance();
    }
    
    /**
     * creates a viewer with the provided statistics
     * @param cell
     * @param x
     * @param y
     * @param width
     * @param height
     */
    public UCViewer2DApp(SharedApp2DImageCell cell, int x, int y, int width, int height) {
        super(new AppGroup(new SimpleControlArb()), true, x, y, width, height, cell);

        // generate task manager
        taskManager = TaskManager.getInstance();
        
        // set up the panel to the loading screen
       /** curPanel = new JPanel(){
        	public void paint(Graphics g){

        	       g.setColor(Color.lightGray);
        	       g.fillRect(0,0, getWidth(), getHeight());
        	       g.setColor(Color.BLACK);
        	       g.setFont(g.getFont().deriveFont(50));
        	       g.drawString("Loading...", getWidth() / 2, getHeight() / 2);
        	}
        };*/
        UCViewer2DListDisplay viewer = new UCViewer2DListDisplay(this);
        curPanel = viewer;
        addMouseListener(viewer);
        addKeyListener(new KeyAdapter(){}); // placeholder for null checker

        setShowing(true);
    }
    
    
    /**
     * Sets up the task manager with information required to access Pivotal Tracker
     * obtained through the wlc
     * @param - user - the name of the user 
     * @param - token - the token of the user
     * @param - IDs - the IDs of the Projects the user wants in our system
     */
    public void setupTaskManager(String user, String token, String IDs){
    	taskManager = TaskManager.getInstance();
    	try{
    		System.out.println("***" + user + " " + token + " " + IDs);
    		taskManager.connectToProjectSite(user, token, IDs);
    	}catch(Exception e){
    		System.err.println("Invalid project IDs in WLC file");
    		System.err.println(e);
    	}
    }
    
    
    /**
     * sets the current project to the provided project
     * @param project
     */
    public void setCurrentProject(IProject project){
    	this.currentProject = project;
    }
    
    /**
     * sets the current task to the provided task
     * @param task
     */
    public void setCurrentTask(ITask task){
    	this.currentTask = task;
    }
    
    /**
     * sets this panel to be a UCViewer2DListDisplay
     */
    public void generateListDisplay(){

		while(getMouseListeners().length > 0){
			MouseListener m = getMouseListeners()[0];
			removeMouseListener(m); 
		}

		while(getKeyListeners().length > 0){
			KeyListener m = getKeyListeners()[0];
			removeKeyListener(m); 
		}
		
        UCViewer2DListDisplay viewer = new UCViewer2DListDisplay(this);
        curPanel = viewer;
        addMouseListener(viewer);
        addKeyListener(new KeyAdapter(){}); // placeholder for null checker

        setSize(width, height);
    }
    
    /**
     * notifies the server that the display has changed to the task list display
     */
    public void notifyListDisplay(){
    	UCViewer2DCellChangeMessage msg = new UCViewer2DCellChangeMessage(this.getCell().getCellID(), "list");
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * sets this panel to be a UCViewer2DViewDisplay
     */
    public void generateViewDisplay(){

		while(getMouseListeners().length > 0){
			MouseListener m = getMouseListeners()[0];
			removeMouseListener(m); 
		}

		while(getKeyListeners().length > 0){
			KeyListener m = getKeyListeners()[0];
			removeKeyListener(m); 
		}
		
        UCViewer2DViewDisplay viewer = new UCViewer2DViewDisplay(this);
        curPanel = viewer;
        addMouseListener(viewer);
        addKeyListener(new KeyAdapter(){}); // placeholder for null checker

        setSize(width, height);
    }
    
    /**
     * notifies the server that the display has changed to the view task display
     */
    public void notifyViewDisplay(){
    	UCViewer2DCellChangeMessage msg = new UCViewer2DCellChangeMessage(this.getCell().getCellID(), "view");
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * sets this panel to be a UCViewer2DEditDisplay
     */
    public void generateEditDisplay(){

		while(getMouseListeners().length > 0){
			MouseListener m = getMouseListeners()[0];
			removeMouseListener(m); 
		}

		while(getKeyListeners().length > 0){
			KeyListener m = getKeyListeners()[0];
			removeKeyListener(m); 
		}
		
		UCViewer2DEditDisplay viewer = new UCViewer2DEditDisplay(this);
        curPanel = viewer;
        addMouseListener(viewer);
        addKeyListener(viewer); 

        setSize(width, height);
    }
    
    /**
     * notifies the server that the display has changed to the edit task display
     */
    public void notifyEditDisplay(){
    	UCViewer2DCellChangeMessage msg = new UCViewer2DCellChangeMessage(this.getCell().getCellID(), "edit");
        ChannelController.getController().sendMessage(msg);
    }
    
    /**
     * sets this panel to be a UCViewer2DProjectSelectDisplay
     */
    public void generateProjectSelectDisplay(){

		while(getMouseListeners().length > 0){
			MouseListener m = getMouseListeners()[0];
			removeMouseListener(m); 
		}

		while(getKeyListeners().length > 0){
			KeyListener m = getKeyListeners()[0];
			removeKeyListener(m); 
		}
		
		UCViewer2DProjectSelectDisplay viewer = new UCViewer2DProjectSelectDisplay(this);
        curPanel = viewer;
        addMouseListener(viewer);
        addKeyListener(new KeyAdapter(){}); // placeholder for null checker
        
        setSize(width, height);
    }

    /**
     * notifies the server that the display has changed to the project selection display
     */
    public void notifyProjectSelectDisplay(){
    	UCViewer2DCellChangeMessage msg = new UCViewer2DCellChangeMessage(this.getCell().getCellID(), "projects");
        ChannelController.getController().sendMessage(msg);
    }
    
    /**
     * sets this panel to be a UCViewer2DLoginDisplay
     */
    public void generateLoginDisplay(){

		while(getMouseListeners().length > 0){
			MouseListener m = getMouseListeners()[0];
			removeMouseListener(m); 
		}

		while(getKeyListeners().length > 0){
			KeyListener m = getKeyListeners()[0];
			removeKeyListener(m); 
		}
		
		UCViewer2DLoginDisplay viewer = new UCViewer2DLoginDisplay(this);
        curPanel = viewer;
        addMouseListener(viewer);
        addKeyListener(viewer);
        
        setSize(width, height);
    }

    /**
     * notifies the server that the display has changed to the login display
     */
    public void notifyLoginDisplay(){
    	UCViewer2DCellChangeMessage msg = new UCViewer2DCellChangeMessage(this.getCell().getCellID(), "login");
        ChannelController.getController().sendMessage(msg);
    }
    
    /**
     * sets the preferred width to the provided size
     * @param preferredWidth
     */
    public void setPreferredWidth(int preferredWidth) {
        this.preferredWidth = preferredWidth;
    }

    /**
     * sets the preferred height to the provided size
     * @param preferredHeight
     */
    public void setPreferredHeight(int preferredHeight) {
        this.preferredHeight = preferredHeight;
    }

    /**
     * Set the size of the application
     * @param width the width of the application
     * @param height the height of the application
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
        curPanel.setSize(width, height);
        repaint();
    }
    
    protected void refreshUI(){
    	setSize(getWidth(), getHeight());
    }
    
    /**
      * Paint contents of window
      */
    @Override
    protected void paint(Graphics2D g) { 
    	if(curPanel != null){
    		curPanel.paint(g);
    	}else{
 	       g.setColor(Color.red);
 	       g.fillRect(0,0, width, height);
    	}
    }
}