package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import javax.vecmath.Matrix4d;
import javax.vecmath.Matrix3f;
import javax.vecmath.Vector3f;
import org.jdesktop.j3d.util.SceneGraphUtil;
import org.jdesktop.lg3d.wonderland.darkstar.client.cell.*;
import org.jdesktop.lg3d.wonderland.darkstar.common.*;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DCellSetup;
import org.jdesktop.lg3d.wonderland.darkstar.common.CellSetup;
import org.jdesktop.lg3d.wonderland.darkstar.client.ExtendedClientChannelListener;
import com.sun.sgs.client.ClientChannel;
import com.sun.sgs.client.SessionId;
import org.jdesktop.lg3d.wonderland.darkstar.client.ChannelController;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.Message;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DCellChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DListChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DTaskSelectMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DProjectSelectChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DProjectSelectMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditNameChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditDateChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditHoursChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditDescriptionChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DLoginNameChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DLoginPasswordChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DListViewer;

import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DSendManagerDataMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DPollManagerDataMessage;

/**
 * UCViewer2DCell.java
 * 
 * The class for the client side of the use case viewer.
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 10/30/08
 * @version 11/19/08 - updated for server changes during activity from other clients
 */
public class UCViewer2DCell extends SharedApp2DImageCell implements ExtendedClientChannelListener {

	private UCViewer2DCellSetup setup;
    
    public UCViewer2DCell(CellID cellID, String channelName, Matrix4d origin) {
        super(cellID, channelName, origin);
    }
    
    @Override
    public void setup(CellSetup setupData) {
        super.setup(setupData);

        setup = (UCViewer2DCellSetup) setupData;
        
        //((UCViewer2DApp) app).setupTaskManager((String)setup.getUserName(), (String)setup.getTokenName(), (String)setup.getProjectID());
        ((UCViewer2DApp) app).setPreferredWidth((int) setup.getPreferredWidth());
        ((UCViewer2DApp) app).setPreferredHeight((int) setup.getPreferredHeight());
        ((UCViewer2DApp) app).setSize((int) setup.getPreferredWidth(), (int) setup.getPreferredHeight());
 
    }    
    
    public void setChannel(ClientChannel channel) {
        this.channel = channel;
    }
    
    public void leftChannel(ClientChannel arg0) {
        // ignore
    }
   
    public void receivedMessage(ClientChannel client, SessionId session, byte[] data) {
        CellMessage msg = Message.extractMessage(data, CellMessage.class);
        
        // check if its a send manager data message
        if (msg instanceof UCViewer2DSendManagerDataMessage) {
        	UCViewer2DSendManagerDataMessage smdm = Message.extractMessage(data, UCViewer2DSendManagerDataMessage.class);
        	String userName = smdm.getUserName();
        	String tokenName = smdm.getTokenName();
        	String projectID = smdm.getProjectID();
        	
        	// setup the task manager
        	System.out.println("calling setup task manager with: " + userName + " " + tokenName + " " + projectID);
        	((UCViewer2DApp) app).setupTaskManager(userName, tokenName, projectID);
        }
        
        // check if its a cell (display) change message
        if (msg instanceof UCViewer2DCellChangeMessage) {
        	UCViewer2DCellChangeMessage sccm = Message.extractMessage(data, UCViewer2DCellChangeMessage.class);
           String display = sccm.getDisplayType();
          
           // check display to change to and generate it
           if(display.equals("list")){
        	   ((UCViewer2DApp) app).generateListDisplay();
           }else if(display.equals("projects")){
        	   ((UCViewer2DApp) app).generateProjectSelectDisplay();
           }else if(display.equals("login")){
        	   ((UCViewer2DApp) app).generateLoginDisplay();
           }else if(display.equals("view")){
        	   ((UCViewer2DApp) app).generateViewDisplay();
           }else if(display.equals("edit")){
        	   ((UCViewer2DApp) app).generateEditDisplay();
           }
        }
        
        // check if list (display) change message
       	if (msg instanceof UCViewer2DListChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DListDisplay){
       			UCViewer2DListChangeMessage sccm = Message.extractMessage(data, UCViewer2DListChangeMessage.class);
                int selection = sccm.getSelectedTask();
                
       			((UCViewer2DListDisplay) ((UCViewer2DApp) app).curPanel).listViewer.selectedObject = selection;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved list display change message but not in list display");
       		}
       	}
       	
       	// check if task select message
       	if (msg instanceof UCViewer2DTaskSelectMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DListDisplay){
       			UCViewer2DTaskSelectMessage sccm = Message.extractMessage(data, UCViewer2DTaskSelectMessage.class);
                int selection = sccm.getSelectedTask();

       			((UCViewer2DListDisplay) ((UCViewer2DApp) app).curPanel).listViewer.selectedObject = selection;
       			((UCViewer2DListDisplay) ((UCViewer2DApp) app).curPanel).setTaskToSelected();
       		}else{
       			System.err.println("ERROR: recieved list display change message but not in list display");
       		}
       	}
        
        // check if project selection (display) change message
       	if (msg instanceof UCViewer2DProjectSelectChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DProjectSelectDisplay){
       			UCViewer2DProjectSelectChangeMessage sccm = Message.extractMessage(data, UCViewer2DProjectSelectChangeMessage.class);
                int selection = sccm.getSelectedProject();
                
       			((UCViewer2DProjectSelectDisplay) ((UCViewer2DApp) app).curPanel).listViewer.selectedObject = selection;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved project select display change message but not in project select display");
       		}
       	}
        
        // check if project select message
       	if (msg instanceof UCViewer2DProjectSelectMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DProjectSelectDisplay){
       			UCViewer2DProjectSelectMessage sccm = Message.extractMessage(data, UCViewer2DProjectSelectMessage.class);
                int selection = sccm.getSelectedProject();
                
       			((UCViewer2DProjectSelectDisplay) ((UCViewer2DApp) app).curPanel).listViewer.selectedObject = selection;
       			((UCViewer2DProjectSelectDisplay) ((UCViewer2DApp) app).curPanel).setProjectToSelected();
       		}else{
       			System.err.println("ERROR: recieved project select message but not in project select display");
       		}
       	}
        
        // check if login (display) name (field) change message
       	if (msg instanceof UCViewer2DLoginNameChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DLoginDisplay){
       			UCViewer2DLoginNameChangeMessage sccm = Message.extractMessage(data, UCViewer2DLoginNameChangeMessage.class);
                String loginText = sccm.getLoginText();
                
       			((UCViewer2DLoginDisplay) ((UCViewer2DApp) app).curPanel).loginText = loginText;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved login display change message but not in login display");
       		}
       	}
        
        // check if login (display) password (field) change message
       	if (msg instanceof UCViewer2DLoginPasswordChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DLoginDisplay){
       			UCViewer2DLoginPasswordChangeMessage sccm = Message.extractMessage(data, UCViewer2DLoginPasswordChangeMessage.class);
                String passwordText = sccm.getPasswordText();
                
       			((UCViewer2DLoginDisplay) ((UCViewer2DApp) app).curPanel).passwordText = passwordText;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved login display change message but not in login display");
       		}
       	}
       	
       	// check if edit (display) name (field) change message
       	if (msg instanceof UCViewer2DEditNameChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DEditDisplay){
       			UCViewer2DEditNameChangeMessage sccm = Message.extractMessage(data, UCViewer2DEditNameChangeMessage.class);
                String nameText = sccm.getNameText();
                
       			((UCViewer2DEditDisplay) ((UCViewer2DApp) app).curPanel).nameText = nameText;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved edit display change message but not in edit display");
       		}
       	}
       	
       	// check if edit (display) date (field) change message
       	if (msg instanceof UCViewer2DEditDateChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DEditDisplay){
       			UCViewer2DEditDateChangeMessage sccm = Message.extractMessage(data, UCViewer2DEditDateChangeMessage.class);
                String dateText = sccm.getDateText();
                
       			((UCViewer2DEditDisplay) ((UCViewer2DApp) app).curPanel).dateText = dateText;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved edit display change message but not in edit display");
       		}
       	}
       	
       	// check if edit (display) hours (field) change message
       	if (msg instanceof UCViewer2DEditHoursChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DEditDisplay){
       			UCViewer2DEditHoursChangeMessage sccm = Message.extractMessage(data, UCViewer2DEditHoursChangeMessage.class);
                String hoursText = sccm.getHoursText();
                
       			((UCViewer2DEditDisplay) ((UCViewer2DApp) app).curPanel).hoursText = hoursText;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved edit display change message but not in edit display");
       		}
       	}
       	
       	// check if edit (display) description (field) change message
       	if (msg instanceof UCViewer2DEditDescriptionChangeMessage) {
       		// check that the display is a list display
       		if ( ((UCViewer2DApp) app).curPanel instanceof UCViewer2DEditDisplay){
       			UCViewer2DEditDescriptionChangeMessage sccm = Message.extractMessage(data, UCViewer2DEditDescriptionChangeMessage.class);
                String descriptionText = sccm.getDescriptionText();
                
       			((UCViewer2DEditDisplay) ((UCViewer2DApp) app).curPanel).descriptionText = descriptionText;
       			((UCViewer2DApp) app).refreshUI();
       		}else{
       			System.err.println("ERROR: recieved edit display change message but not in edit display");
       		}
       	}
    }
    
}