package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import org.jdesktop.lg3d.wonderland.appshare.AppWindowGraphics2DApp;
import org.jdesktop.lg3d.wonderland.darkstar.client.ChannelController;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.ITask;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditNameChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditDateChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditHoursChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditDescriptionChangeMessage;

import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DButtonBar;

/**
 * UCViewer2DEditDisplay.java
 * 
 * The panel for the user to edit a selected task
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/23/08
 *
 */
public class UCViewer2DEditDisplay extends JPanel implements MouseListener, KeyListener {

	/**
	 * the application this panel is for
	 */
	private UCViewer2DApp app;
	
	/**
	 * the button bar
	 */
	private UCViewer2DButtonBar buttonBar = new UCViewer2DButtonBar(3);
	private String[] buttonTexts = {"Update", "Assign", "Back"};
	
	/**
	 * the ui elements
	 */
	protected String nameText = "";
	protected boolean nameTextSelected = false;
	protected String dateText = "";
	protected boolean dateTextSelected = false;
	protected String hoursText = "";
	protected boolean hoursTextSelected = false;
	protected String descriptionText = "";
	protected boolean descriptionTextSelected = false;
	
	/**
	 * sets up this panel for the provided app
	 * @param viewerApp
	 */
	public UCViewer2DEditDisplay(UCViewer2DApp viewerApp){
		// set globals
		this.app = viewerApp;
		setBackground(Color.white);
		
		// set up buttons
		try{
			buttonBar.setupButtons(getWidth(), getHeight(), buttonTexts);
		}catch(Exception e){
			e.printStackTrace();
		}
		
		// fill the ui elements with the current task
		if(app.currentTask != null){
			// pull name
			nameText = app.currentTask.getName();

			// pull type
			Object pulledValue = app.currentTask.getProperties()[0];
			if(pulledValue != null) dateText = (String) pulledValue;
			
			// pull estimate
			pulledValue = app.currentTask.getProperties()[1];
			if(pulledValue != null) hoursText = (String) pulledValue;
			
			// pull description
			pulledValue = app.currentTask.getProperties()[2];
			if(pulledValue != null) descriptionText = (String) pulledValue;
		}
	}

    /**
     * Set the size of the application
     * @param width the width of the application
     * @param height the height of the application
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
        try{
        	buttonBar.setupButtons(width, height, buttonTexts);
        }catch(Exception e){
        	e.printStackTrace();
        }
        repaint();
    }
	
	@Override
	public void paint(Graphics g){
		super.paint(g);
		Font tempFont = new Font("Courier New", Font.BOLD, 40);
		g.setFont(tempFont);

		// NOTE! if no value assigned the controller package will return null so all fields check for that

		// paint borders or fill if selected
		/** name */
		if(!nameTextSelected){
			// paint title border
			g.setColor(Color.black);
			g.drawRect(getWidth() / 10, getHeight() / 10, 3 * (getWidth() / 10), getHeight() / 10);
		}else{
			g.setColor(Color.lightGray);
			g.fillRect(getWidth() / 10, getHeight() / 10, 3 * (getWidth() / 10), getHeight() / 10);
		}
		/** date */
		if(!dateTextSelected){
			// paint date border
			g.setColor(Color.black);
			g.drawRect(4 * (getWidth() / 10), getHeight() / 10, (3 * (getWidth() / 10)) - 5, getHeight() / 10);
		}else{
			g.setColor(Color.lightGray);
			g.fillRect(4 * (getWidth() / 10), getHeight() / 10, (3 * (getWidth() / 10)) - 5,getHeight() / 10);
		}
		/** hours */
		if(!hoursTextSelected){
			// paint hours border
			g.setColor(Color.black);
			g.drawRect(7 * (getWidth() / 10), getHeight() / 10, (2 * (getWidth() / 10)) - 5, getHeight() / 10);
		}else{
			g.setColor(Color.lightGray);
			g.fillRect(7 * (getWidth() / 10), getHeight() / 10, (2 * (getWidth() / 10)) - 5, getHeight() / 10);
		}
		/** description */
		if(!descriptionTextSelected){
			// paint description border 
			g.setColor(Color.black);
			g.drawRect(getWidth() / 10, 4 * (getHeight() / 10), 8 * (getWidth() / 10), 4 * (getHeight() / 10));
		}else{
			g.setColor(Color.lightGray);
			g.fillRect(getWidth() / 10, 4 * (getHeight() / 10), 8 * (getWidth() / 10), 4 * (getHeight() / 10));
		}
		
		if(app.currentTask != null){
			g.setColor(Color.black);
			// draw title
			g.drawString("Task: " + nameText, 10 + (getWidth() / 10), (getHeight() / 10) + 35);
			
			// draw date
			g.drawString("Created: " + dateText , 10 + (4 * (getWidth() / 10)), (getHeight() / 10) + 35);
			
			// draw estimated (or actual if complete) hours
			g.drawString("Hours: " + hoursText , 10 + (7 * (getWidth() / 10)), (getHeight() / 10) + 35);
			
			// draw assigned user
			String assignedUser = "";
			Object pulledValue = app.currentTask.getProperties()[5];
			if(pulledValue != null) assignedUser = (String) pulledValue;
			g.drawString("Assigned to: " + assignedUser , 10 + (getWidth() / 10), (2 * (getHeight() / 10)) + 35);
			
			// draw creator user
			String creatorUser = "";
			pulledValue = app.currentTask.getProperties()[3];
			if(pulledValue != null) creatorUser = (String) pulledValue;
			g.drawString("Created by: " + creatorUser , 10 + (5 * (getWidth() / 10)), (2 * (getHeight() / 10)) + 35);
			
			// draw description
			String description = descriptionText;
			int gap = 35;
			if(description.length() > (49 * 6)) description = description.substring(0, 49 * 6);
			while(description.length() > 49){
				String segment = description.substring(0, 49);
				g.drawString(segment , 10 + (getWidth() / 10), (4 * (getHeight() / 10)) + gap);
				description = description.substring(49); gap += 50; // increment
			}
			g.drawString(description , 10 + (getWidth() / 10), (4 * (getHeight() / 10)) + gap);
		}else{
			System.err.println("Error: Current Task is null");
		}

		// paint buttons
		buttonBar.paint(g);
	}
	
	public void mouseClicked(MouseEvent e){
		// reset texts
		nameTextSelected = false;
		dateTextSelected = false;
		hoursTextSelected = false;
		descriptionTextSelected = false;
		
		// get mouse point location
		Point p = e.getPoint();
		
		// check if name
		if((p.x >= 1 * (getWidth() / 10))&&(p.x <= 4 * (getWidth() / 10))){
			if((p.y >= 1 * (getHeight() / 10))&&(p.y <= 2 * (getHeight() / 10))){
				nameTextSelected = true;
				app.refreshUI();
				return;
			}
		}

		// check if date
		if((p.x >= 4 * (getWidth() / 10))&&(p.x <= 7 * (getWidth() / 10))){
			if((p.y >= 1 * (getHeight() / 10))&&(p.y <= 2 * (getHeight() / 10))){
				dateTextSelected = true;
				app.refreshUI();
				return;
			}
		}

		// check if hours
		if((p.x >= 7 * (getWidth() / 10))&&(p.x <= 9 * (getWidth() / 10))){
			if((p.y >= 1 * (getHeight() / 10))&&(p.y <= 2 * (getHeight() / 10))){
				hoursTextSelected = true;
				app.refreshUI();
				return;
			}
		}

		// check if description
		if((p.x >= 1 * (getWidth() / 10))&&(p.x <= 9 * (getWidth() / 10))){
			if((p.y >= 4 * (getHeight() / 10))&&(p.y <= 8 * (getHeight() / 10))){
				descriptionTextSelected = true;
				app.refreshUI();
				return;
			}
		}
		
		// find button clicked
		int choice = buttonBar.buttonClicked(p);
		
		System.out.println("Button found " + choice);
		
		switch(choice){
		case -1: 
			app.refreshUI();
			return; // by default
		case 0:
			// set the new properties
			app.currentTask.setProperty("name", nameText);
			app.currentTask.setProperty("story_type", dateText);
			app.currentTask.setProperty("estimate", hoursText);
			app.currentTask.setProperty("description", descriptionText);

			// return to the view display
			app.notifyViewDisplay();
			app.generateViewDisplay();
			break;
		case 1:
			// TODO assign button response
			break;
		case 2:
			app.notifyViewDisplay();
			app.generateViewDisplay();
			break;
		}
		
	 }
	
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e) {}
	
	public void keyPressed(KeyEvent e){
		int code = e.getKeyCode();
		
		if(code == KeyEvent.VK_SHIFT) return; // weed out shift key
		
		// check for name
		if(nameTextSelected){
			if((code == KeyEvent.VK_BACK_SPACE)||(code == KeyEvent.VK_DELETE)){
				if(nameText.length() > 0) nameText = nameText.substring(0, nameText.length() - 1);
			}else{
				nameText = nameText + e.getKeyChar();
			}
			notifyNameChange(); // notify name text change
			app.refreshUI();
		}
		
		// check for date
		if(dateTextSelected){
			if((code == KeyEvent.VK_BACK_SPACE)||(code == KeyEvent.VK_DELETE)){
				if(dateText.length() > 0) dateText = dateText.substring(0, dateText.length() - 1);
			}else{
				dateText = dateText + e.getKeyChar();
			}
			notifyDateChange(); // notify date text change
			app.refreshUI();
		}

		// check for hours
		if(hoursTextSelected){
			if((code == KeyEvent.VK_BACK_SPACE)||(code == KeyEvent.VK_DELETE)){
				if(hoursText.length() > 0) hoursText = hoursText.substring(0, hoursText.length() - 1);
			}else{
				hoursText = hoursText + e.getKeyChar();
			}
			notifyHoursChange(); // notify hours text change
			app.refreshUI();
		}

		// check for description
		if(descriptionTextSelected){
			if((code == KeyEvent.VK_BACK_SPACE)||(code == KeyEvent.VK_DELETE)){
				if(descriptionText.length() > 0) descriptionText = descriptionText.substring(0, descriptionText.length() - 1);
			}else{
				descriptionText = descriptionText + e.getKeyChar();
			}
			notifyDescriptionChange(); // notify description text change
			app.refreshUI();
		}
	}
	public void keyReleased(KeyEvent e){}
    public void keyTyped(KeyEvent e){}
    

    /**
     * notifies the server to set the name field to the current name text
     */
    public void notifyNameChange(){
    	UCViewer2DEditNameChangeMessage msg = new UCViewer2DEditNameChangeMessage(app.getCell().getCellID(), nameText);
        ChannelController.getController().sendMessage(msg);
    }
    
    /**
     * notifies the server to set the date field to the current date text
     */
    public void notifyDateChange(){
    	UCViewer2DEditDateChangeMessage msg = new UCViewer2DEditDateChangeMessage(app.getCell().getCellID(), dateText);
        ChannelController.getController().sendMessage(msg);
    }
    
    /**
     * notifies the server to set the hours field to the current hours text
     */
    public void notifyHoursChange(){
    	UCViewer2DEditHoursChangeMessage msg = new UCViewer2DEditHoursChangeMessage(app.getCell().getCellID(), hoursText);
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * notifies the server to set the description field to the current description text
     */
    public void notifyDescriptionChange(){
    	UCViewer2DEditDescriptionChangeMessage msg = new UCViewer2DEditDescriptionChangeMessage(app.getCell().getCellID(), descriptionText);
        ChannelController.getController().sendMessage(msg);
    }
}