package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import javax.swing.*;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import org.jdesktop.lg3d.wonderland.appshare.AppWindowGraphics2DApp;
import org.jdesktop.lg3d.wonderland.darkstar.client.ChannelController;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DLoginNameChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DLoginPasswordChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DPollManagerDataMessage;

import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DButtonBar;

/**
 * UCViewer2DLoginDisplay.java
 * 
 * The panel for the user to login from
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/6/08
 *
 */
public class UCViewer2DLoginDisplay extends JPanel implements MouseListener, KeyListener {

	/**
	 * the application this panel is for
	 */
	private UCViewer2DApp app;

	/**
	 * the button bar
	 */
	private UCViewer2DButtonBar buttonBar = new UCViewer2DButtonBar(2);
	private String[] texts = {"Login", "Back"};
	
	/**
	 * the text area info
	 */
	protected String loginText = "";
	protected boolean loginTextSelected = false;
	protected String passwordText = "";
	protected boolean passwordTextSelected = false;
	
	/**
	 * sets up this panel for the provided app
	 * @param viewerApp
	 */
	public UCViewer2DLoginDisplay(UCViewer2DApp viewerApp){
		// set globals
		this.app = viewerApp;
		setBackground(Color.white);
		
		// set up buttons
		try{
			buttonBar.setupButtons(getWidth(), getHeight(), texts);
		}catch(Exception e){
			e.printStackTrace();
		}
	}

    /**
     * Set the size of the application
     * @param width the width of the application
     * @param height the height of the application
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
		try{
			buttonBar.setupButtons(width, height, texts);
		}catch(Exception e){
			e.printStackTrace();
		}
        repaint();
    }
	
	@Override
	public void paint(Graphics g){
		super.paint(g);

		Font titleFont = new Font("Courier New", Font.BOLD, 40);
		g.setFont(titleFont);
		g.drawString("Name: ", 2 * (getWidth() / 5), 6 * (getHeight() / 20));

		// paint name border
		g.setColor(Color.black);
		g.drawRect(2 * (getWidth() / 5), 7 * (getHeight() / 20), 1 * (getWidth() / 5), 1 * (getHeight() / 20));

		Font textFont = g.getFont();
		textFont = textFont.deriveFont(Font.BOLD, 20);
		g.setFont(textFont);
		// fill name with whats there
		g.drawString(loginText, 5 + (2 * (getWidth() / 5)), 15 * (getHeight() / 40));

		g.setFont(titleFont);
		g.drawString("Password: ", 2 * (getWidth() / 5), 10 * (getHeight() / 20));
		
		// paint password border
		g.setColor(Color.black);
		g.drawRect(2 * (getWidth() / 5), 11 * (getHeight() / 20), 1 * (getWidth() / 5), 1 * (getHeight() / 20));

		g.setFont(textFont);
		// fill password with whats there
		g.drawString(passwordText, 5 + (2 * (getWidth() / 5)), 23 * (getHeight() / 40));

		// paint buttons
		buttonBar.paint(g);
	}
	
	public void mouseClicked(MouseEvent e){
		// reset texts
		loginTextSelected = false;
		passwordTextSelected = false;
		
		// get mouse point location
		Point p = e.getPoint();
		
		// check if login
		if((p.x >= 2 * (getWidth() / 5))&&(p.x <= 3 * (getWidth() / 5))){
			if((p.y >= 7 * (getHeight() / 20))&&(p.y <= 8 * (getHeight() / 20))){
				loginTextSelected = true;
				return;
			}
		}
		
		// check if password
		if((p.x >= 2 * (getWidth() / 5))&&(p.x <= 3 * (getWidth() / 5))){
			if((p.y >= 11 * (getHeight() / 20))&&(p.y <= 12 * (getHeight() / 20))){
				passwordTextSelected = true;
				return;
			}
		}
		
		// find button clicked
		int choice = buttonBar.buttonClicked(p);
		
		switch(choice){
		case -1: return; // by default
		case 0:
			// TODO login button response
			
			// for now,
			notifySelectedProjectChange();
			
			app.notifyListDisplay();
			app.generateListDisplay();
			break;
		case 1:
			app.notifyListDisplay();
			app.generateListDisplay();
			break;
		}
	 }
	
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e) {}
	
	public void keyPressed(KeyEvent e){
		int code = e.getKeyCode();
		
		// check for text
		if(loginTextSelected){
			if((code == KeyEvent.VK_BACK_SPACE)||(code == KeyEvent.VK_DELETE)){
				if(loginText.length() > 0) loginText = loginText.substring(0, loginText.length() - 1);
			}else{
				loginText = loginText + e.getKeyChar();
			}
			notifyLoginChange(); // notify login text change
			app.refreshUI();
		}
		
		// check for password
		if(passwordTextSelected){
			if((code == KeyEvent.VK_BACK_SPACE)||(code == KeyEvent.VK_DELETE)){
				if(passwordText.length() > 0) passwordText = passwordText.substring(0, passwordText.length() - 1);
			}else{
				passwordText = passwordText + e.getKeyChar();
			}
			notifyPasswordChange(); // notify password text change
			app.refreshUI();
		}
	}
	public void keyReleased(KeyEvent e){}
    public void keyTyped(KeyEvent e){}

    /**
     * notifies the server to set the login field to the current login text
     */
    public void notifyLoginChange(){
    	UCViewer2DLoginNameChangeMessage msg = new UCViewer2DLoginNameChangeMessage(app.getCell().getCellID(), loginText);
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * notifies the server to set the password field to the current password text
     */
    public void notifyPasswordChange(){
    	UCViewer2DLoginPasswordChangeMessage msg = new UCViewer2DLoginPasswordChangeMessage(app.getCell().getCellID(), passwordText);
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * notifies the server that the selected project has changed to the selected index
     */
    public void notifySelectedProjectChange(){
    	UCViewer2DPollManagerDataMessage msg = new UCViewer2DPollManagerDataMessage(app.getCell().getCellID());
        ChannelController.getController().sendMessage(msg);
    }
}
