package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import javax.swing.*;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import org.jdesktop.lg3d.wonderland.appshare.AppWindowGraphics2DApp;
import org.jdesktop.lg3d.wonderland.darkstar.client.ChannelController;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.TaskManager;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.IProject;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DProjectSelectChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DProjectSelectMessage;

import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DButtonBar;
import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DListViewer;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.IProject;

import java.util.ArrayList;

/**
 * UCViewer2DProjectSelectDisplay.java
 * 
 * The panel for displaying the list of projects for the user to select from
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/6/08
 *
 */
public class UCViewer2DProjectSelectDisplay extends JPanel implements MouseListener {


	/**
	 * the application this panel is for
	 */
	private UCViewer2DApp app;
	
	/**
	 * the project list viewer
	 */
	protected UCViewer2DListViewer listViewer;

	/**
	 * the button bar
	 */
	private UCViewer2DButtonBar buttonBar = new UCViewer2DButtonBar(2);
	private String[] texts = {"Select","Back"};
	
	/**
	 * sets up this panel for the provided app
	 * @param viewerApp
	 */
	public UCViewer2DProjectSelectDisplay(UCViewer2DApp viewerApp){
		// set globals
		this.app = viewerApp;
		setBackground(Color.white);
		
		// set up viewer
		listViewer = new UCViewer2DListViewer<IProject>("Projects", app.taskManager.getAllProjectsForUser());
		
		// set up buttons
		listViewer.setupButtons(getWidth(), getHeight());
		try{
			buttonBar.setupButtons(getWidth(), getHeight(), texts);
		}catch(Exception e){
			e.printStackTrace();
		}
	}

    /**
     * Set the size of the application
     * @param width the width of the application
     * @param height the height of the application
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
		listViewer.setupButtons(getWidth(), getHeight());
		try{
			buttonBar.setupButtons(width, height, texts);
		}catch(Exception e){
			e.printStackTrace();
		}
        repaint();
    }
	
	@Override
	public void paint(Graphics g){
		super.paint(g);
		// paint the list
		listViewer.paint(g, getWidth(), getHeight());
		
		// paint buttons
		buttonBar.paint(g);
	}
	
	public void mouseClicked(MouseEvent e){
		// get mouse point location
		Point p = e.getPoint();
		
		// find selection clicked
		int selectionChoice = listViewer.selectionMade(p, getHeight(), getWidth());
		if(selectionChoice != -1){
			listViewer.selectedObject = selectionChoice;
			notifySelectedProjectChange(); // notify the server
			System.out.println("Selected project switched to " + listViewer.getSelectedObjectName());
			app.refreshUI();
			return;
		}
		
		// find if list viewer button clicked
		int listButtonChoice = listViewer.buttonClicked(p);
		switch(listButtonChoice){
		case 0:
			if(listViewer.selectedObject > 0) listViewer.selectedObject--;
			if(listViewer.selectedObject < listViewer.lowerBound) listViewer.lowerBound = listViewer.selectedObject;
			app.refreshUI();
			break;
		case 1:
			if(listViewer.selectedObject < listViewer.objectList.size() - 1) listViewer.selectedObject++;

			int maxDisplayCount = 0;
			int tempHeightValue = 7 * (getHeight() / 10);
			while(tempHeightValue > 45){
				tempHeightValue -= 45;
				maxDisplayCount++;
			}
			if(listViewer.selectedObject - listViewer.lowerBound >= maxDisplayCount) listViewer.lowerBound++;
			app.refreshUI();
			break;
		}
		
		// find button clicked
		int buttonChoice = buttonBar.buttonClicked(p);

		switch(buttonChoice){
		case -1: return; // by default
		case 0:
			if(listViewer.selectedObject != -1){
				// set current project
				notifySelectProject();
				setProjectToSelected();

				app.notifyListDisplay();
				app.generateListDisplay();
			}
			break;
		case 1:
			app.notifyListDisplay();
			app.generateListDisplay();
			break;
		}
		
	 }
	
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e) {}

	/**
	 * sets the project in the viewer to the selected project in the list
	 */
	protected void setProjectToSelected(){
		app.setCurrentProject((IProject) listViewer.objectList.get(listViewer.selectedObject));
	}
	
    /**
     * notifies the server that the selected project has changed to the selected index
     */
    public void notifySelectedProjectChange(){
    	UCViewer2DProjectSelectChangeMessage msg = new UCViewer2DProjectSelectChangeMessage(app.getCell().getCellID(), listViewer.selectedObject);
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * notifies the server that to set the project to the selected project index
     */
    public void notifySelectProject(){
    	UCViewer2DProjectSelectMessage msg = new UCViewer2DProjectSelectMessage(app.getCell().getCellID(), listViewer.selectedObject);
        ChannelController.getController().sendMessage(msg);
    }
}
