package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import javax.swing.*;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import org.jdesktop.lg3d.wonderland.appshare.AppWindowGraphics2DApp;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.ITask;

import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DButtonBar;

/**
 * UCViewer2DViewDisplay.java
 * 
 * The panel for the user to view a selected task
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/23/08
 *
 */
public class UCViewer2DViewDisplay extends JPanel implements MouseListener {

	/**
	 * the application this panel is for
	 */
	private UCViewer2DApp app;

	/**
	 * the button bar
	 */
	private UCViewer2DButtonBar buttonBar = new UCViewer2DButtonBar(2);
	private String[] texts = {"Edit", "Back"};
	
	/**
	 * sets up this panel for the provided app
	 * @param viewerApp
	 */
	public UCViewer2DViewDisplay(UCViewer2DApp viewerApp){
		// set globals
		this.app = viewerApp;
		setBackground(Color.white);
		
		// set up buttons
		try{
			buttonBar.setupButtons(getWidth(), getHeight(), texts);
		}catch(Exception e){
			e.printStackTrace();
		}
	}

    /**
     * Set the size of the application
     * @param width the width of the application
     * @param height the height of the application
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
		try{
			buttonBar.setupButtons(width, height, texts);
		}catch(Exception e){
			e.printStackTrace();
		}
        repaint();
    }
	
	@Override
	public void paint(Graphics g){
		super.paint(g);
		Font tempFont = new Font("Courier New", Font.BOLD, 40);
		g.setFont(tempFont);

		// NOTE! if no value assigned the controller package will return null so all fields check for that

		// paint description border 
		g.setColor(Color.black);
		g.drawRect(getWidth() / 10, 4 * (getHeight() / 10), 8 * (getWidth() / 10), 4 * (getHeight() / 10));
		
		Object[] info;
		info = app.currentTask.getProperties();
		
		if(app.currentTask != null){
			// draw title
			g.setColor(Color.black);
			g.drawString("Task: " + app.currentTask.getName(), 10 + (getWidth() / 10), (getHeight() / 10) + 35);
			
			// draw date
			g.setColor(Color.black);
			String creationDate = "";
			Object pulledValue = app.currentTask.getProperties()[0];
			if(pulledValue != null) creationDate = (String) pulledValue;
			g.drawString("Type: " + creationDate , 10 + (4 * (getWidth() / 10)), (getHeight() / 10) + 35);
			
			// draw estimated (or actual if complete) hours
			g.setColor(Color.black);
			String estimatedHours = "";
			pulledValue = app.currentTask.getProperties()[1];
			if(pulledValue != null) estimatedHours = (String) pulledValue;
			g.drawString("Est. Hours: " + estimatedHours , 10 + (7 * (getWidth() / 10)), (getHeight() / 10) + 35);
			
			// draw assigned user
			g.setColor(Color.black);
			String assignedUser = "";
			pulledValue = app.currentTask.getProperties()[5];
			if(pulledValue != null) assignedUser = (String) pulledValue;
			g.drawString("Assigned to: " + assignedUser , 10 + (getWidth() / 10), (2 * (getHeight() / 10)) + 35);
			
			// draw creator user
			g.setColor(Color.black);
			String creatorUser = "";
			pulledValue = app.currentTask.getProperties()[3];
			if(pulledValue != null) creatorUser = (String) pulledValue;
			g.drawString("Created by: " + creatorUser , 10 + (5 * (getWidth() / 10)), (2 * (getHeight() / 10)) + 35);
			
			// draw description
			g.setColor(Color.black);
			String description = "";
			pulledValue = app.currentTask.getProperties()[2];
			if(pulledValue != null) description = (String) pulledValue;
			int gap = 35;
			if(description.length() > (49 * 6)) description = description.substring(0, 49 * 6);
			while(description.length() > 49){
				String segment = description.substring(0, 49);
				g.drawString(segment , 10 + (getWidth() / 10), (4 * (getHeight() / 10)) + gap);
				description = description.substring(49); gap += 50; // increment
			}
			g.drawString(description , 10 + (getWidth() / 10), (4 * (getHeight() / 10)) + gap);
		}else{
			System.err.println("Error: Current Task is null");
		}

		// paint buttons
		buttonBar.paint(g);
	}
	
	public void mouseClicked(MouseEvent e){
		// get mouse point location
		Point p = e.getPoint();
		
		// find button clicked
		int choice = buttonBar.buttonClicked(p);
		
		System.out.println("Button found " + choice);
		
		switch(choice){
		case -1: return; // by default
		case 0:
			app.notifyEditDisplay();
			app.generateEditDisplay();
			break;
		case 1:
			app.notifyListDisplay();
			app.generateListDisplay();
			break;
		}
	 }
	
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e) {}
}
