package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;
import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;


/**
 * UCViewer2DCellChangeMessage.java
 * 
 * The class for communicating changes between the client cell and the server cell GLO
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/19/08
 *
 */
public class UCViewer2DCellChangeMessage extends CellMessage {

	/**
	 * the type of display
	 *  either: "list" "projects" "login" "view" "edit"
	 */
	protected String displayType;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DCellChangeMessage(CellID cellID, String displayType){
		super(cellID);
		this.displayType = displayType;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DCellChangeMessage(){}
	
	/**
	 * @return the display type
	 */
    public String getDisplayType() {
        return this.displayType;
    }

    /**
     * sets the display type to the provided type
     * @param displayType
     */
    public void setDisplayType(String displayType) {
        this.displayType = displayType;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.displayType));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.displayType = DataString.value(data);
	}

}
