package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import javax.vecmath.Matrix4f;
import org.jdesktop.lg3d.wonderland.darkstar.common.setup.SharedApp2DImageCellSetup;
import org.jdesktop.lg3d.wonderland.usecaseviewer.client.UCViewer2DApp;
/**
 * UCViewer2DCellSetup.java
 * 
 * The cell setup file for the client and server side code of the 2D use case viewer.
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 10/30/08
 */
public class UCViewer2DCellSetup extends SharedApp2DImageCellSetup {


    private static final int DEFAULT_WIDTH = 1500;
    private static final int DEFAULT_HEIGHT = 800;
    private double preferredWidth = DEFAULT_WIDTH;
    private double preferredHeight = DEFAULT_HEIGHT;
    private String tokenName = "dog";
    private String userName = "cat";
    private String projectID = "mouse";
    
    public UCViewer2DCellSetup() {
        this(null, null, null);
    }

    public UCViewer2DCellSetup(String appName, Matrix4f viewRectMat, String clientClassName) {
        super(appName, viewRectMat, clientClassName);
    }
    

    /**
     * Return the classname of the AppWindowGraphics2DApp subclass
     */
    @Override
    public String getClientClassname() {
        return UCViewer2DApp.class.getName();
    }
    /*
     * Set the preferred width of the whiteboard
     * @param preferredWidth the preferred width in pixels
     */
    public void setPreferredWidth(double preferredWidth) {
        this.preferredWidth = preferredWidth;
    }

    /*
     * Get the preferred width of the whiteboard
     * @return the preferred width, in pixels
     */
    public double getPreferredWidth() {
        return preferredWidth;
    }

    /*
     * Set the preferred height of the whiteboard
     * @param preferredHeight the preferred height, in pixels
     */
    public void setPreferredHeight(double preferredHeight) {
        this.preferredHeight = preferredHeight;
    }

    /*
     * Get the preferred height of the whiteboard
     * @return the preferred height, in pixels
     */
    public double getPreferredHeight() {
        return preferredHeight;
    }
    
    
	/* set the user name for the task manager
	* @param userName the user name on Pivotal Tracker
	*/
	public void setUserName(String userName){
		this.userName = userName;
		System.out.println("***" + this.userName);
	}

	/**
	* get the user name that logs into pivotal tracker
	* @return the user name on Pivotal Tracker
	*/
	public String getUserName(){
		return userName;
	}

	/**
	* set the token for the task manager
	* @param tokenName the token for the user on Pivotal Tracker
	*/
	public void setTokenName(String tokenName){
		this.tokenName = tokenName;
		System.out.println("***" + this.tokenName);
	}

	/**
	* get the token the user logs into pivotal tracker with
	* @return the token for Pivotal Tracker
	*/
	public String getTokenName(){
		return tokenName;
	}
    
    /**
     * gets the project IDs to use with pivotal tracker
     * @return the project IDs for pivotal Tracker
     */
    public String getProjectID(){
    	return projectID;
    }
    
    /**
     * Sets the project IDs to use with pivotal tracker
     * @param - ProjectIDs - the Project IDs to parse and use with pivotal tracker
     */
    public void setProjectID(String projectID){
    	this.projectID = projectID;
    	System.out.println("***" + this.projectID);
    }
}
