package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DEditDateChangeMessage.java
 * 
 * The class for communicating changes in the creation date field of the edit display to the server
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DEditDateChangeMessage extends CellMessage {

	/**
	 * the date text
	 */
	protected String dateText;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DEditDateChangeMessage(CellID cellID, String dateText){
		super(cellID);
		this.dateText = dateText;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DEditDateChangeMessage(){}
	
	/**
	 * @return the date text
	 */
    public String getDateText() {
        return this.dateText;
    }

    /**
     * sets the date text to the provided date
     * @param dateText
     */
    public void setDateText(String dateText) {
        this.dateText = dateText;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.dateText));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.dateText = DataString.value(data);
	}

}
