package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DEditDescriptionChangeMessage.java
 * 
 * The class for communicating changes in the description field of the edit display to the server
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DEditDescriptionChangeMessage extends CellMessage {

	/**
	 * the description text
	 */
	protected String descriptionText;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DEditDescriptionChangeMessage(CellID cellID, String descriptionText){
		super(cellID);
		this.descriptionText = descriptionText;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DEditDescriptionChangeMessage(){}
	
	/**
	 * @return the description text
	 */
    public String getDescriptionText() {
        return this.descriptionText;
    }

    /**
     * sets the description Text to the provided description
     * @param descriptionText
     */
    public void setDescriptionText(String descriptionText) {
        this.descriptionText = descriptionText;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.descriptionText));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.descriptionText = DataString.value(data);
	}

}
