package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DEditHoursChangeMessage.java
 * 
 * The class for communicating changes in the estimated hours field of the edit display to the server
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DEditHoursChangeMessage extends CellMessage {

	/**
	 * the hours text
	 */
	protected String hoursText;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DEditHoursChangeMessage(CellID cellID, String hoursText){
		super(cellID);
		this.hoursText = hoursText;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DEditHoursChangeMessage(){}
	
	/**
	 * @return the hours text
	 */
    public String getHoursText() {
        return this.hoursText;
    }

    /**
     * sets the hours text to the provided date
     * @param hoursText
     */
    public void setHoursText(String hoursText) {
        this.hoursText = hoursText;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.hoursText));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.hoursText = DataString.value(data);
	}

}
