package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DEditNameChangeMessage.java
 * 
 * The class for communicating changes in the name field of the edit display to the server
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DEditNameChangeMessage extends CellMessage {

	/**
	 * the name text
	 */
	protected String nameText;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DEditNameChangeMessage(CellID cellID, String nameText){
		super(cellID);
		this.nameText = nameText;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DEditNameChangeMessage(){}
	
	/**
	 * @return the name text
	 */
    public String getNameText() {
        return this.nameText;
    }

    /**
     * sets the name text to the provided type
     * @param nameText
     */
    public void setNameText(String nameText) {
        this.nameText = nameText;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.nameText));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.nameText = DataString.value(data);
	}

}
