package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DLoginNameChangeMessage.java
 * 
 * The class for communicating changes in the login name field of the login display to the server
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DLoginNameChangeMessage extends CellMessage {

	/**
	 * the login text
	 */
	protected String loginText;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DLoginNameChangeMessage(CellID cellID, String loginText){
		super(cellID);
		this.loginText = loginText;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DLoginNameChangeMessage(){}
	
	/**
	 * @return the login Text
	 */
    public String getLoginText() {
        return this.loginText;
    }

    /**
     * sets the login text to the provided login
     * @param loginText
     */
    public void setLoginText(String loginText) {
        this.loginText = loginText;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.loginText));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.loginText = DataString.value(data);
	}

}
