package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DLoginPasswordChangeMessage.java
 * 
 * The class for communicating changes in the login password field of the login display to the server
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DLoginPasswordChangeMessage extends CellMessage {

	/**
	 * the password Text
	 */
	protected String passwordText;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DLoginPasswordChangeMessage(CellID cellID, String passwordText){
		super(cellID);
		this.passwordText = passwordText;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DLoginPasswordChangeMessage(){}
	
	/**
	 * @return the password Text
	 */
    public String getPasswordText() {
        return this.passwordText;
    }

    /**
     * sets the password Text to the provided login
     * @param passwordText
     */
    public void setPasswordText(String passwordText) {
        this.passwordText = passwordText;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.passwordText));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.passwordText = DataString.value(data);
	}

}
