package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DProjectSelectMessage.java
 * 
 * The class for communicating a selection of a project in the project list display 
 * between the client cell and the server cell GLO
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/03/08
 *
 */
public class UCViewer2DProjectSelectMessage extends CellMessage {

	/**
	 * the index of the project that is being selected in the list
	 */
	protected int selectedProject;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DProjectSelectMessage(CellID cellID, int selectedProject){
		super(cellID);
		this.selectedProject = selectedProject;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DProjectSelectMessage(){}
	
	/**
	 * @return the index of the project
	 */
    public int getSelectedProject() {
        return this.selectedProject;
    }

    /**
     * sets the index of the task to the provided index
     * @param displayType
     */
    public void setSelectedProject(int selectedProject) {
        this.selectedProject = selectedProject;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.selectedProject + ""));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.selectedProject = Integer.parseInt(DataString.value(data));
	}

}
