package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DSendManagerDataMessage.java
 * 
 * The message for SENDING THE DEETS.
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 3/17/09
 */
public class UCViewer2DSendManagerDataMessage extends CellMessage {

	private String userName;
	private String tokenName;
	private String projectID;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DSendManagerDataMessage(CellID cellID, String userName, String tokenName, String projectID){
		super(cellID);
		this.userName = userName;
		this.tokenName = tokenName;
		this.projectID = projectID;
	}

	/**
	 * default constructor
	 */
	public UCViewer2DSendManagerDataMessage(){}
	
	/**
	 * @return the user name of the pivotal TRACKAAA
	 */
    public String getUserName() {
        return this.userName;
    }

    /**
     * sets the index of the task to the provided index
     * @param userName
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }

	/**
	 * @return the token name of the pivotal TRACKAAA
	 */
    public String getTokenName() {
        return this.tokenName;
    }

    /**
     * sets the index of the task to the provided index
     * @param tokenName
     */
    public void setTokenName(String tokenName) {
        this.tokenName = tokenName;
    }

	/**
	 * @return the project id of the pivotal TRACKAAA
	 */
    public String getProjectID() {
        return this.projectID;
    }

    /**
     * sets the index of the task to the provided index
     * @param projectID
     */
    public void setProjectID(String projectID) {
        this.projectID = projectID;
    }


	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.userName + ""));
	    dataElements.add(new DataString(this.tokenName + ""));
	    dataElements.add(new DataString(this.projectID + ""));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.userName = DataString.value(data);
		this.tokenName = DataString.value(data);
		this.projectID = DataString.value(data);
	}

}
