package org.jdesktop.lg3d.wonderland.usecaseviewer.common;

import java.nio.ByteBuffer;

import org.jdesktop.lg3d.wonderland.darkstar.common.CellID;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.DataString;

/**
 * UCViewer2DTaskSelectMessage.java
 * 
 * The class for communicating the selection of a task 
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 12/04/08
 *
 */
public class UCViewer2DTaskSelectMessage extends CellMessage {

	/**
	 * the index of the task that is being selected in the list
	 */
	protected int selectedTask;
	
	/**
	 * sets up the new cell change message with the provided information
	 * @param cellID
	 * @param message
	 */
	public UCViewer2DTaskSelectMessage(CellID cellID, int selectedTask){
		super(cellID);
		this.selectedTask = selectedTask;
	}
	
	/**
	 * default constructor
	 */
	public UCViewer2DTaskSelectMessage(){}
	
	/**
	 * @return the index of the task
	 */
    public int getSelectedTask() {
        return this.selectedTask;
    }

    /**
     * sets the index of the task to the provided index
     * @param displayType
     */
    public void setSelectedTask(int selectedTask) {
        this.selectedTask = selectedTask;
    }

	@Override
	protected void populateDataElements() {
	    super.populateDataElements();
	    dataElements.add(new DataString(this.selectedTask + ""));
	}
	  
	@Override    
	protected void extractMessageImpl(ByteBuffer data) {   
		super.extractMessageImpl(data);    
		this.selectedTask = Integer.parseInt(DataString.value(data));
	}

}
