package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;



import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import com.mysql.jdbc.Connection;
import com.mysql.jdbc.Statement;

/**
 * This object handles communication with an external database. It is not usable since
 * It works with XPlanner and that is installed locally on my PC but this is just
 * another example of what can be done with the controller package
 * 
 * @author Ryan Moak
 */


public class DatabaseCommunicator {
	
	//The connection for this class.  Used by JDBC
	static Connection conn;
	
	
	/**
	 * Connects to a MYSQL server and returns a Statement that can be used to query
	 * the database
	 * @return Statement to use for querying the connected database
	 */
	public static Statement connect(){
        try{
            String userName = "xplanner";
            String password = "xp";
            String url = "jdbc:mysql://rmoak613.res.wpi.net:3306/xplanner";
            Class.forName ("com.mysql.jdbc.Driver").newInstance ();
            conn = (Connection) DriverManager.getConnection (url, userName, password);
            return (Statement) conn.createStatement();
        }
        catch (Exception e){
           // System.err.println ("Cannot connect to database server");
        	e.printStackTrace();
        }
       return null;
    }
    
    /**
     * disconnects from the provided statement
     * @param s the statement to disconnect
     */
    public static void disconnect(Statement s){
        try{
            s.close();
        }
        catch (Exception e){
            System.err.println ("Cannot connect to database server");
        }
        if (conn != null){
                try{
                    conn.close ();
                }
                catch (Exception e) { /* ignore close errors */ }
        }
    }
    
    /**
     * Provided with a project ID for XPlanner, populates an ArrayList of Tasks by querying
     * the database
     */
    public static void populateTasks(int id, ArrayList<ITask> tasks){
    	Statement st = connect();
    	ArrayList<Integer> iterationIds = new ArrayList<Integer>();
    	ArrayList<Integer> storyIds = new ArrayList<Integer>();
    	try {
    		//first gets project with that ID and adds all iteration IDs to a list
			st.executeQuery("select * from iteration where project_id = '" + id + "';");
			ResultSet rs = st.getResultSet();
			while (rs.next()){
				iterationIds.add(new Integer(rs.getInt("id")));
			}
			
			//searches through the iterations to get the stories contained within
			//the iteration
			for (int i = 0; i < iterationIds.size(); i++){
				st.executeQuery("select * from story where iteration_id = '" 
						+ Integer.parseInt(iterationIds.get(i).toString()) + "';");
				rs = st.getResultSet();
				while (rs.next()){
					storyIds.add(new Integer(rs.getInt("id")));
				}
			}
			
			//for each story, gets the task information
			for (int i = 0; i < storyIds.size(); i++){
				st.executeQuery("select * from task where story_id = '" 
						+ Integer.parseInt(storyIds.get(i).toString()) + "';");
				rs = st.getResultSet();
				while (rs.next()){
					
					tasks.add(new DBTask(rs.getString("name"), rs.getInt("id")));
				}
			}
			
			disconnect(st);
		
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			disconnect(st);
			e.printStackTrace();
		}
    }
    
    
    /**
     * Gets the properties of a task with the provided ID
     *@param id for the task
     *@return an array of Objects for each of the properties, description, estimated hours,
     *and type
     */
    public static Object[] getTaskProperties(int id){
    	Statement st = connect();
    	Object[] props = new Object[3];
    	try {
			st.executeQuery("Select * from task where id = '" + id + "' ;");
			ResultSet rs = st.getResultSet();
			while (rs.next()){
				props[0] = rs.getString("description");
				props[1] = rs.getString("estimated_hours");
				props[2] = rs.getString("type");
			}
			disconnect(st);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			disconnect(st);
			e.printStackTrace();
		}
		return props;
    
    }

    /**
     * Updates a property of a specific task
     * @param parameter - the paremeter to update must be type, estimated_hours, or description
     * @param newValue - a String representing the new value
     * @param ID - the ID of the task to modify
     */
	public static void updateTaskProperty(String parameter, String newValue, int ID) {
		Statement s = connect();
		try {
			s.executeUpdate("Update task Set " + parameter + " = '" + newValue + "' where id = '" + ID + "' ;");
			disconnect(s);
		} catch (SQLException e){
			disconnect(s);
			e.printStackTrace();
		}
		
	}

	/**
	 * Used for creating new tasks.  This is for creating an iteration if a project has none
	 * @param projectID - the ID of the Project to add to
	 * @param newID - the ID of the iteration that will be created
	 * 
	 */
	public static void createNewIteration(int projectId, int newID ){
		Statement st = connect();
		try {
			st.executeUpdate("Insert into iteration (id, last_update, project_id, name, " +
					"description, start_date, end_date, status, days_worked) " +
					"values ('" + newID + "', '2008-12-03 20:57:45', '" + projectId + "', 'iteration" + projectId + "', 'an iteration', '2009-01-01', '2013-01-01', '1', '0') ;");
			disconnect(st);
		} catch (SQLException e){
			disconnect(st);
			e.printStackTrace();
		}
	}
	
	/**
	 * Creates a new Story within an Iteration.  Used for creating new Tasks
	 * @param iterationId - the ID of the iteration to add this story to
	 * @param newID - the new ID of story to create
	 */
	public static void createNewStory(int iterationId, int newID ){
		Statement st = connect();
		try {
			st.executeUpdate("Insert into story (id, last_update, name, description, " +
					"iteration_id, tracker_id, estimated_hours, priority, customer_id, status," +
					"original_estimated_hours, disposition, postponed_hours, it_start_estimated_hours, orderNo) " +
					"values ('" + newID + "', '2008-12-03 19:20:46', 'story" + iterationId + "', 'a story', '"  
					+ iterationId + "', '0', '0', '4', '1', 'd', NULL,  'p', '0', '0', '1') ;");
			disconnect(st);
		} catch (SQLException e){
			disconnect(st);
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Creates a new task with a specified name and ID, inside a story with provided ID
	 * @param storyId - the ID of the story to add the task to
	 * @param newID - the new ID of the task to add
	 * @param name - a string representing the name of the story
	 */
	public static boolean createNewTask(int storyId, int newID, String name){
		Statement st = connect();
		try {
			st.executeUpdate("Insert into task (id, last_update,  name, type, description, " +
					"acceptor_id, created_date, estimated_hours, original_estimate, is_complete, story_id, disposition) " +
					"values ('" + newID + "', '2009-01-07 21:30:01', '" + name + "', 'Feature', ' ', '0', '2009-01-01', '0', '0', '0', '" + storyId + "', 'p') ;");
			disconnect(st);
			return true;
		} catch (SQLException e){
			disconnect(st);
			e.printStackTrace();
			return false;
		}
	}
	
	
	/**
	 * Returns an ID for an iteration within a specific Project
	 * @param projectID - the ID of the project to get an iteration from
	 * @return an Int for the interation ID
	 */
	public static int getIterationID(int projectID){
			Statement st = connect();
			try {
				st.executeQuery("Select * from iteration where project_id = '" + projectID + "' ;");
				ResultSet rs = st.getResultSet();
				if(rs.next()){
					return rs.getInt("id");
				}
				else {
					return -1;
				}
			} catch (SQLException e){
				disconnect(st);
				e.printStackTrace();
			}
			return -1;
		}
	
	/**
	 * Gets a story ID from the Iteration with provided ID
	 * @param the ID of the iteration to get the story ID from
	 * @return an int for the story ID
	 */
	public static int getStoryID(int iterationID){
		Statement st = connect();
		try {
			st.executeQuery("Select * from story where iteration_id = '" + iterationID + "' ;");
			ResultSet rs = st.getResultSet();
			if(rs.next()){
				return rs.getInt("id");
			}
			else {
				return -1;
			}
		} catch (SQLException e){
			disconnect(st);
			e.printStackTrace();
		}
		return -1;
	}
	
	/**
	 * Checks to see if there are any current items in the database with a provided ID.
	 * Used for creating new stories so that only original IDs are used.
	 * @param newID - the ID to check 
	 * @return a boolean if the ID is new, false if the ID already exists 
	 */
	public static boolean checkID(int newID) {
		Statement s = connect();
		try {
			s.executeQuery("Select * from iteration where id = '" + newID + "' ;");
			ResultSet rs = s.getResultSet();
			if (rs.next()){
				return false;
			}
			
			s.executeQuery("Select * from note where id = '" + newID + "' ;");
			rs = s.getResultSet();
			if (rs.next()){
				return false;
			}
			
			s.executeQuery("Select * from person where id = '" + newID + "' ;");
			rs = s.getResultSet();
			if (rs.next()){
				return false;
			}
			
			s.executeQuery("Select * from story where id = '" + newID + "' ;");
			rs = s.getResultSet();
			if (rs.next()){
				return false;
			}
			
			s.executeQuery("Select * from task where id = '" + newID + "' ;");
			rs = s.getResultSet();
			if (rs.next()){
				return false;
			}
			
			s.executeQuery("Select * from project where id = '" + newID + "' ;");
			rs = s.getResultSet();
			if (rs.next()){
				return false;
			}

			disconnect(s);
			
		} catch (SQLException e){
			disconnect(s);
			e.printStackTrace();
		}
		return true;
	
	}

	/**
	 * Removes a task with the provided ID
	 * @param id - the ID of the task to remove
	 * @return a boolean, true if the task was removed, false otherwise.
	 */
	public static boolean removeTask(int id) {
		Statement st = connect();
		try {
			st.executeUpdate("Delete from task where id = '" + id + "' ;");
			disconnect(st);
			return true;
		
		} catch (SQLException e){
			disconnect(st);
			e.printStackTrace();
			return false;
		}
		
	}
    
    
    
}
