package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;


import java.util.Collection;
/**
 * The IProject interface.  In order to configure our code to work with a new repository
 * there must be a class that implements this.
 * @author Ryan Moak
 *
 */
public interface IProject {

	/**
	 * Gets the tasks for this project
	 * @return - An ArrayList of ITask objects
	 */
	public Collection<ITask> getAllTasks();
	
	/**
	 * Gets a specific task from this project specified by name
	 * @param - name the name of the task to retrieve
	 * @return - ITask from this project with the specified name
	 */
	public ITask getTask(String name);
	
	/**
	 * Removes a task from this Project
	 * @param - name - the name of the task to remove
	 * @return - true if the task was successfully removed, false otherwise
	 */
	public boolean removeTask(String name);
	
	/**
	 * Modifies the properties of a task in this project
	 * @param - task - the ITask to modify
	 * @param - propertyToModify - a String representing the property to Modify
	 * @param - newValue - new value of the property to save
	 */
	public void modifyTask(ITask task, String propertyToModify, String newValue) throws Exception;
	
	/**
	 * Gets the name of the project
	 * @return - a string representing the name of the project
	 */
	public String getProjectName();
	
	/**
	 * Adds a task to this project
	 * @param - name - the name of the task to add
	 */
	public boolean addTask(String name);
	
	/**
	 * Returns a string representation of this object
	 * @return name of the project
	 */
 	public String toString();
	
	
	
}
