package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * PTConnector.java
 * 
 * The connector implementation for the Pivotal Tracker database.
 * 
 * @author Ryan Moak
 * @author James Johnson
 * @author Matt Duval
 *
 */
public class PTConnector implements IConnector{

	// the projects and ids
	ArrayList<IProject> projects;
	ArrayList<Integer> projectIDs;

	// the communicator
	HTTPCommunicator comm;

	// the factories for the xml
	DOMParser parser = new DOMParser();
	XPathFactory xfactory = XPathFactory.newInstance();
	XPath path = xfactory.newXPath();

	/**
	 * set up the connector
	 */
	public PTConnector(){
		comm = new HTTPCommunicator();
		projectIDs = new ArrayList<Integer>();
	}


	/**
	 *Used to setup the PTConnector with variables from the WLC file
	 */
	public boolean connect(String username, String password, String path) {
		comm.setUsername(username);
		comm.setToken(password);
		
		Pattern p = Pattern.compile("([0-9]+:*)*");
		Matcher m = p.matcher(path);
		if(!m.matches()){
			return false;
		}
		
		
		String[] IDs = path.split(":", 0);
		
		for(int i = 0; i < IDs.length; i++){
			addProjectID(Integer.parseInt(IDs[i]));
			System.out.println("***" + IDs[i]);
		}
		
		return true;
	}

	/**
	 * disconnect, does nothing right now
	 */
	public void disconnect() {

	}

	/**
	 * get all the projects from this connection
	 */
	public ArrayList<IProject> getProjects() {
		if(projects == null){
			try{
				projects = new ArrayList<IProject>();
				/*String urlString = "http://130.215.241.155:8081/ProjectIDs.xml";

				// send and get request
				String information = comm.sendGetRequest(urlString, null, false);

				// set up parser and document
				parser.parse(new InputSource(new StringReader(information)));
				Document infoDoc = parser.getDocument();

				// parse information 
				String tempUserName = path.evaluate("/xml/user", infoDoc); // get name
				String tempToken = path.evaluate("/xml/Access", infoDoc); // get token
				int projIDCount = Integer.parseInt(path.evaluate("count(/xml/Projects/ID)", infoDoc));
				for(int i = 1; i <= projIDCount; i++){
					projectIDs.add(Integer.parseInt(path.evaluate("/xml/Projects/ID[" + i + "]", infoDoc)));
				}**/
				/*
				projectIDs.add(new Integer(7406)); // TODO temp

				// fill pulled info
				comm.setToken("32c967674edcdfc86a51e138f1fa3b2e"); // comm.setToken(tempToken); TODO temp
				comm.setUsername("Matt Duval"); //comm.setUsername(tempUserName); TODO temp
	*/

				String projectString = "http://www.pivotaltracker.com/services/v1/projects/";


				for (int i = 0; i < projectIDs.size(); i++){
					// send and get request for the project ID
					String projectData = comm.sendGetRequest(projectString + projectIDs.get(i).toString(), null, true);

					// set up parser and document
					parser.parse(new InputSource(new StringReader(projectData)));
					Document projDoc = parser.getDocument();

					if (projectData != null){
						// parse project data
						String tempProjName = path.evaluate("/response/project/name", projDoc);

						// create and add project
						projects.add(new PTProject(tempProjName, projectIDs.get(i), comm));
					}
				}
			}catch(SAXException e){
				e.printStackTrace();
			}catch(XPathExpressionException e){
				e.printStackTrace();
			}catch(IOException e){
				e.printStackTrace();
			}
		}
		return projects;
	}

	/**
	 * Adds a project ID to the list of project IDs
	 * @param ID - the project ID to add to the project
	 */
	private void addProjectID(int ID){
		projectIDs.add(new Integer(ID));
	}
	
	
	
	
	
}

