package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;
/**
 * The Task implementation for the Pivotal Tracker database
 * @author Ryan Moak
 *
 */
public class PTTask implements ITask{

	private String name; //name of task
	private int id; //id of task
	private int projID; //projID that manages this task
	private Object[] props; //the properties of this task
	private HTTPCommunicator comm; //the HTTP communicator for this object


	/**
	 * Constructor for a PTTask
	 * @param n - the name of the task
	 * @param i - the id for this task
	 * @param properties - the attributes of this task
	 * @param commu - the HTTP communicator for this task
	 */
	public PTTask(String n, int i, int project, Object[] properties, HTTPCommunicator commu){
		name = n;
		id = i;
		projID = project;
		comm = commu;
		if (properties != null){
			props = new Object[properties.length];
			for (int j = 0; j < properties.length; j++){
				props[j] = properties[j];
			}
		}
		else {
			props = new Object[]{"","","","","",""};
		}
	}

		/**
		 * Sets the name of the task
		 * @param n - the name of the task
		 */
		private void setName(String n){
			name = n;
		}


		/**
		 * Gets the name of this task
		 */
		public String getName() {
			return name;
		}

		public String toString(){
			return getName();
		}

		/**
		 * Gets the properties associated with this task
		 */
		public Object[] getProperties() {
			if (props == null){
				props = new Object[]{"","","","","",""};
			}

			return props;
		}

		/**
		 * gets the ID of this task
		 */
		public int getID(){
			return id;
		}

		/**
		 * Gets the project ID of the project that contains this task
		 */
		private int getProjID(){
			return projID;
		}


		/**
		 * Changes a property of this task to a specified value
		 * @param - parameter - the parameter to modify, must be "name",
		 * 						"story_type", "estimate", "description", 
		 * 						"requested_by", "current_state", or "owned_by"
		 * @param - newValue - the new Value of the specified parameter
		 */
		public void setProperty(String parameter, String newValue) {
			if (checkParams(parameter)){

				String updateURL = "http://www.pivotaltracker.com/services/v1/projects/"+ getProjID() + "/stories/" + getID();

				String request = "<story><" + parameter + ">" + newValue + "</" + parameter + "></story>";

				String data = null;
				try {
				 data = comm.postData(updateURL, request, "PUT");

				} catch (Exception e) {

					e.printStackTrace();
				}

				boolean success = false;
				//check to make sure response is successful
				if (data!= null){
					success = true;
				}
				else {
					success = false;
				}


				if (success){
					if(parameter.equals("name")){
						setName(newValue);
					}

					else if (parameter.equals("story_type")){
						props[0] = newValue;
					}

					else if (parameter.equals("estimate")){
						props[1] = newValue;
					}

					else if (parameter.equals("description")){
						props[2] = newValue;
					}

					else if (parameter.equals("requested_by")){
						props[3] = newValue;
					}

					else if (parameter.equals("current_state")){
						props[4] = newValue;
					}

					else if (parameter.equals("owned_by")){
						props[5] = newValue;
					}
				}

			}

		}

		/**
		 * checks to make sure the parameter provided to setProperty is valid
		 * @param - str - the parameter to check
		 */
		private boolean checkParams(String str){
			if (str.equals("name") || str.equals("story_type") || str.equals("estimate") || str.equals("description")
					|| str.equals("owned_by") || str.equals("requested_by") || str.equals("current_state")){
				return true;
			}

			return false;
		}

	}
