package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;


import java.util.Collection;

/**
 * Main Class for the Controller package that communicates with the client
 * Meant to be a singleton class
 * @author Ryan Moak
 *
 */


public class TaskManager {
	
	//an instance of TaskManager 
	private final static TaskManager INSTANCE = new TaskManager();

	
	//the IConnector to use
	IConnector myConnector;
	
	/**
	 * Constructor for this object.  
	 */
	protected TaskManager(){
		myConnector = new PTConnector();
	}
	
	/**
	 * Returns an instance of TaskManager
	 * @return an instance of TaskManager
	 */
	public synchronized static TaskManager getInstance(){
		return INSTANCE;
	}
	
	
	/**
	 * Returns a collection of all the projects for the user specified in IController
	 */
	public ArrayList<IProject> getAllProjectsForUser(){
		return (ArrayList<IProject>)myConnector.getProjects();
	}
	
	/**
	 * Connects to the project site.  Not functional
	 */
	public void connectToProjectSite(String user, String pass, String path) throws Exception{
		boolean test = myConnector.connect(user, pass, path);
		if (!test){	
			throw new Exception("Project IDs did not match correct format");
		}
	}
	
	
	
}
