package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.net.URL;
import java.net.URLConnection;
import java.io.InputStreamReader;


/**
 * 
 * The Project implementation for our Text controller
 * 
 * @author Ryan Moak
 * @version base
 * 
 * @author James Johnson
 * @version fixed bug in getAllTasks
 *
 */
public class TextProject implements IProject{
	//the name of this project
	String myName;
	
	//the list of tasks
	ArrayList<ITask> tasks;

	/**
	 * Constructor for this object
	 * @param name - the name of this TextProject
	 */
	public TextProject(String name){
		myName = name;
		tasks = null;
		populateTasks();
	}

	/**
	 * Populates the list of tasks that is contained in this porject by accessing
	 * the repository.txt file
	 */
	public void populateTasks(){
		if (tasks == null){
			tasks = new ArrayList<ITask>();
			BufferedReader reader;
			URL url;
			URLConnection connection;
			//Like TextConnector this will need to be set
			//String urlString = "http://path-to/repository.txt";
			try {
				url = new URL(urlString);
				connection = url.openConnection();
				reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
				try {
					String current = reader.readLine();
					while (current != null){
						if (current.equals("TASKS")){
							while(true){
								String atask = reader.readLine();
								if (atask.equals("ENDTASKS")){
									break;
								}
								else{
									int index = atask.indexOf(' ');
									String name = atask.substring(0, index);
									if (name.equals(getProjectName())){
										ITask newTask = createTaskFromString(atask.substring(index+1));
										tasks.add(newTask);
									}
								}
							}
						}
						current = reader.readLine();
					}
				
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
	
		
	}
	}
	

	/**
	 * Returns the name of this project
	 * @return String representing the name of this project
	 */
	public String getProjectName() {
		return myName;
	}

	/**
	 * Gets a task with a specified name from the list of tasks
	 * @param -name - the name of the task to acquire
	 * @return ITask with the specified name or null if non existant
	 */
	public ITask getTask(String name) {
		if (tasks == null){
			populateTasks();
		}
		ArrayList<ITask> tasks = getAllTasks();
		for (int i = 0; i < tasks.size(); i++){
			if (tasks.get(i).getName().equalsIgnoreCase(name)){
				return tasks.get(i);
			}		
		}
		return null;
	}

	/**
	 * Modifies a task contained in this project
	 * @param - task - the ITask to modify
	 * @param propertyToModify - the property of the task to modify
	 * @param newValue - the newValue of the property to input into the task
	 */
	public void modifyTask(ITask task, String propertyToModify, String newValue) {
		task.setProperty(propertyToModify, newValue);
	}

	/**
	 * Removes a task with the provided name from the list of tasks managed
	 * by this project
	 * @param name - the name of the project to remove
	 * @return boolean - true if removed successfully, false if ITask could not be found
	 */
	public boolean removeTask(String name) {
		if (tasks == null){
			populateTasks();
		}
		for (int i = 0; i < tasks.size(); i++){
			if (tasks.get(i).getName().equals(name)){
				ITask taskToRemove= tasks.get(i);
				tasks.remove(taskToRemove);
				return true;
			}
		}
		return false;
	}

	
	/**
	 * Creates a task from a string in repository.txt
	 * @param - str - the String that contains the task information
	 * @return ITask with the information from the repository.txt contained within
	 */
	private ITask createTaskFromString(String str){
		int index = 0;
		index = str.indexOf(' ');
		String name = str.substring(0, index);
		int index2 = str.indexOf(' ', index+1);
		String units = str.substring(index+1, index2);
		index = str.indexOf(' ', index2+1);
		String assigned = str.substring(index2+1, index);
		String status = str.substring(index+1);
		ITask task = new TextTask(name, new Object[]{units, assigned, status});
		return task;
	}

	/**
	 * Add task function, was not implemented in this build
	 */
	public boolean addTask(String name) {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * Not implemented in this build either
	 */
	public ArrayList<ITask> getAllTasks() {
		// TODO Auto-generated method stub
		return null;
	}


}
