
package org.jdesktop.lg3d.wonderland.usecaseviewer.server;

import javax.media.j3d.*;
import javax.vecmath.Matrix4d;
import javax.vecmath.Matrix4f;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DCellSetup;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DCellChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DListChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DTaskSelectMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DProjectSelectChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DProjectSelectMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditNameChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditDateChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditHoursChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DEditDescriptionChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DLoginNameChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DLoginPasswordChangeMessage;
import org.jdesktop.lg3d.wonderland.darkstar.server.cell.*;
import org.jdesktop.lg3d.wonderland.darkstar.server.setup.*;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.CellMessage;
import org.jdesktop.lg3d.wonderland.darkstar.common.messages.Message;
import org.jdesktop.lg3d.wonderland.darkstar.server.CellMessageListener;

import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DSendManagerDataMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DPollManagerDataMessage;

import java.util.HashSet;
import java.util.Set;
import com.sun.sgs.app.ClientSession;

/**
 * UCViewer2DCellGLO.java
 * 
 * The server side code for the Use Case 2D Viewer
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 10/30/08
 * @version 11/19/08 - added updating for client activity
 */
public class UCViewer2DCellGLO extends SharedApp2DImageCellGLO implements BeanSetupGLO, CellMessageListener {

    public UCViewer2DCellGLO() {
        this(null, null, null, null);
    }

    public UCViewer2DCellGLO(Bounds bounds, String appName, Matrix4d cellOrigin, Matrix4f viewRectMat) {
        super(bounds, appName, cellOrigin, viewRectMat, UCViewer2DCellGLO.class.getName());
    }
    
    @Override
    public UCViewer2DCellSetup getSetupData() {
       return new UCViewer2DCellSetup();
    }
    
    private String userName;
    private String tokenName;
    private String projectID;
    
    public void setupCell(CellGLOSetup setupData) {
        super.setupCell((BasicCellGLOSetup<?>) setupData);
        
       BasicCellGLOSetup<UCViewer2DCellSetup> setup = (BasicCellGLOSetup<UCViewer2DCellSetup>)setupData;
       
       		UCViewer2DCellSetup stp = setup.getCellSetup();
       		userName = stp.getUserName();
       		tokenName = stp.getTokenName();
       		projectID = stp.getProjectID();
        
            super.setBounds(new BoundingBox());
           
    }
    
    public void reconfigureCell(CellGLOSetup setupData) {
        setupCell(setupData);
    }
    
    public CellGLOSetup getCellGLOSetup() {
        return new BasicCellGLOSetup<UCViewer2DCellSetup>(getBounds(),
                getOrigin(), getClass().getName(), getSetupData());
    }
    
    @Override
    public String getClientCellClassName() {
        return "org.jdesktop.lg3d.wonderland.usecaseviewer.client.UCViewer2DCell";
    }


    @Override
    public void openChannel() {
        this.openDefaultChannel();
    }
    
    public void receivedMessage(ClientSession client, CellMessage message) {

    	// check if poll manager data message
    	if(message instanceof UCViewer2DPollManagerDataMessage){
    		// recieve the message
    		UCViewer2DPollManagerDataMessage pmdm = (UCViewer2DPollManagerDataMessage)message;
    		
    		// generate the message to send out
    		UCViewer2DSendManagerDataMessage msg = new UCViewer2DSendManagerDataMessage();
    		System.out.println("server sending: " + userName + " " + tokenName + " " + projectID);
    		msg.setUserName(userName);
    		msg.setTokenName(tokenName);
    		msg.setProjectID(projectID);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            getCellChannel().send(sessions, msg.getBytes());
            sessions.remove(client);
    	}
    	
    	// check if cell change message
    	if(message instanceof UCViewer2DCellChangeMessage){
    		// recieve the message
    		UCViewer2DCellChangeMessage sccm = (UCViewer2DCellChangeMessage)message;
            String display = sccm.getDisplayType();
            
            // generate the message to send out
            UCViewer2DCellChangeMessage msg = new UCViewer2DCellChangeMessage();
            msg.setDisplayType(display);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
    	}

        // check if list (display) change message
       	if (message instanceof UCViewer2DListChangeMessage) {
       		// recieve the message
       		UCViewer2DListChangeMessage sccm = (UCViewer2DListChangeMessage)message;
            int task = sccm.getSelectedTask();
            
            // generate the message to send out
            UCViewer2DListChangeMessage msg = new UCViewer2DListChangeMessage();
            msg.setSelectedTask(task);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
       	
       	// check if task select message
       	if (message instanceof UCViewer2DTaskSelectMessage) {
       		// recieve the message
       		UCViewer2DTaskSelectMessage sccm = (UCViewer2DTaskSelectMessage)message;
            int task = sccm.getSelectedTask();
            
            // generate the message to send out
            UCViewer2DTaskSelectMessage msg = new UCViewer2DTaskSelectMessage();
            msg.setSelectedTask(task);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
        
        // check if project selection (display) change message
       	if (message instanceof UCViewer2DProjectSelectChangeMessage) {
       		// recieve the message
       		UCViewer2DProjectSelectChangeMessage sccm = (UCViewer2DProjectSelectChangeMessage)message;
            int project = sccm.getSelectedProject();
            
            // generate the message to send out
            UCViewer2DProjectSelectChangeMessage msg = new UCViewer2DProjectSelectChangeMessage();
            msg.setSelectedProject(project);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
        
        // check if project select message
       	if (message instanceof UCViewer2DProjectSelectMessage) {
       		// recieve the message
       		UCViewer2DProjectSelectMessage sccm = (UCViewer2DProjectSelectMessage)message;
            int project = sccm.getSelectedProject();
            
            // generate the message to send out
            UCViewer2DProjectSelectMessage msg = new UCViewer2DProjectSelectMessage();
            msg.setSelectedProject(project);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
        
        // check if login (display) name (field) change message
       	if (message instanceof UCViewer2DLoginNameChangeMessage) {
       		// recieve the message
       		UCViewer2DLoginNameChangeMessage sccm = (UCViewer2DLoginNameChangeMessage)message;
            String loginText = sccm.getLoginText();
            
            // generate the message to send out
            UCViewer2DLoginNameChangeMessage msg = new UCViewer2DLoginNameChangeMessage();
            msg.setLoginText(loginText);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
        
        // check if login (display) password (field) change message
       	if (message instanceof UCViewer2DLoginPasswordChangeMessage) {
       		// recieve the message
       		UCViewer2DLoginPasswordChangeMessage sccm = (UCViewer2DLoginPasswordChangeMessage)message;
            String passwordText = sccm.getPasswordText();
            
            // generate the message to send out
            UCViewer2DLoginPasswordChangeMessage msg = new UCViewer2DLoginPasswordChangeMessage();
            msg.setPasswordText(passwordText);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
       	
       	// check if edit (display) name (field) change message
       	if (message instanceof UCViewer2DEditNameChangeMessage) {
       		// recieve the message
       		UCViewer2DEditNameChangeMessage sccm = (UCViewer2DEditNameChangeMessage)message;
            String text = sccm.getNameText();
            
            // generate the message to send out
            UCViewer2DEditNameChangeMessage msg = new UCViewer2DEditNameChangeMessage();
            msg.setNameText(text);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
       	
       	// check if edit (display) date (field) change message
       	if (message instanceof UCViewer2DEditDateChangeMessage) {
       		// recieve the message
       		UCViewer2DEditDateChangeMessage sccm = (UCViewer2DEditDateChangeMessage)message;
            String text = sccm.getDateText();
            
            // generate the message to send out
            UCViewer2DEditDateChangeMessage msg = new UCViewer2DEditDateChangeMessage();
            msg.setDateText(text);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
       	
       	// check if edit (display) hours (field) change message
       	if (message instanceof UCViewer2DEditHoursChangeMessage) {
       		// recieve the message
       		UCViewer2DEditHoursChangeMessage sccm = (UCViewer2DEditHoursChangeMessage)message;
            String text = sccm.getHoursText();
            
            // generate the message to send out
            UCViewer2DEditHoursChangeMessage msg = new UCViewer2DEditHoursChangeMessage();
            msg.setHoursText(text);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
       	
       	// check if edit (display) description (field) change message
       	if (message instanceof UCViewer2DEditDescriptionChangeMessage) {
	       	// recieve the message
       		UCViewer2DEditDescriptionChangeMessage sccm = (UCViewer2DEditDescriptionChangeMessage)message;
            String text = sccm.getDescriptionText();
            
            // generate the message to send out
            UCViewer2DEditDescriptionChangeMessage msg = new UCViewer2DEditDescriptionChangeMessage();
            msg.setDescriptionText(text);

            // send the message
            Set<ClientSession> sessions = new HashSet<ClientSession>(getCellChannel().getSessions());
            sessions.remove(client);
            getCellChannel().send(sessions, msg.getBytes());
       	}
    	
    }

        

}

