package org.jdesktop.lg3d.wonderland.usecaseviewer.ui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;

/**
 * UCViewer2DButtonBar.java
 * 
 * A generalized class for a UI component that represents the set of buttons at the bottom of each display.
 * 
 * @author James Johnson
 * @version 3/1/09
 *
 */
public class UCViewer2DButtonBar {

	/**
	 * the button info
	 */
	private int[][] buttonBounds;
	private String[] buttonTexts;
	
	/**
	 * sets up the new button bar with the provided number of buttons
	 * @param num
	 */
	public UCViewer2DButtonBar(int num){
		buttonBounds = new int[num][4];
		buttonTexts = new String[num];
	}

	/**
	 * sets up the buttons for this display
	 * @throws Exception 
	 */
	public void setupButtons(int width, int height, String[] texts) throws Exception{
		if(texts.length != buttonBounds.length) throw new Exception("Error setting up buttons: incorrect texts array size");
		if(buttonBounds.length >= 1){
			buttonBounds[0][0] = width / 10;
			buttonBounds[0][1] = 17 * (height / 20);
			buttonBounds[0][2] = 2 * (width / 10);
			buttonBounds[0][3] = 19 * (height / 20);
			buttonTexts[0] = texts[0];
		}
		if(buttonBounds.length >= 2){
			buttonBounds[1][0] = 5 * (width / 20);
			buttonBounds[1][1] = 17 * (height / 20);
			buttonBounds[1][2] = 7 * (width / 20);
			buttonBounds[1][3] = 19 * (height / 20);
			buttonTexts[1] = texts[1];
		}
		if(buttonBounds.length >= 3){
			buttonBounds[2][0] = 4 * (width / 10);
			buttonBounds[2][1] = 17 * (height / 20);
			buttonBounds[2][2] = 5 * (width / 10);
			buttonBounds[2][3] = 19 * (height / 20);
			buttonTexts[2] = texts[2];
		}
		if(buttonBounds.length >= 4){
			buttonBounds[3][0] = 11 * (width / 20);
			buttonBounds[3][1] = 17 * (height / 20);
			buttonBounds[3][2] = 13 * (width / 20);
			buttonBounds[3][3] = 19 * (height / 20);
			buttonTexts[3] =  texts[3];
		}
		if(buttonBounds.length >= 5){
			buttonBounds[4][0] = 7 * (width / 10);
			buttonBounds[4][1] = 17 * (height / 20);
			buttonBounds[4][2] = 8 * (width / 10);
			buttonBounds[4][3] = 19 * (height / 20);
			buttonTexts[4] =  texts[4];
		}
	}
	
	/**
	 * checks which button is clicked, if any
	 * @param p
	 * @return the index of the button in the list, or -1 if none
	 */
	public int buttonClicked(Point p){
		int choice = -1; // default
		
		// find the button
		for(int i = 0; i < buttonBounds.length; i++){
			if((p.x >= buttonBounds[i][0])&&(p.x <= buttonBounds[i][2])){
				if((p.y >= buttonBounds[i][1])&&(p.y <= buttonBounds[i][3])){
					choice = i;
				}
			}
		}
		
		return choice; // return found, if any
	}
	
	/**
	 * paint this component on the provided graphoics
	 * @param g
	 */
	public void paint(Graphics g){
		g.setColor(Color.black);
		Font butFont = g.getFont();
		butFont = butFont.deriveFont(Font.BOLD, 30);
		g.setFont(butFont);
		// cycle through each and paint
		for(int i = 0; i < buttonBounds.length; i++){
			int w =  buttonBounds[i][2] - buttonBounds[i][0];
			int h = buttonBounds[i][3] - buttonBounds[i][1];
			g.drawRect(buttonBounds[i][0], buttonBounds[i][1], w, h);
			
			g.drawString(buttonTexts[i], buttonBounds[i][0] + 5, buttonBounds[i][1] + ( 3 * (h / 4)));
		}
	}
}
