package org.jdesktop.lg3d.wonderland.usecaseviewer.ui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;

import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.IProject;

/**
 * UCViewer2DListViewer.java
 * 
 * A generalized class for a UI component that represents the list viewer and controls for it, for displays with lists.
 * 
 * @author James Johnson
 * @version 3/1/09
 *
 */
public class UCViewer2DListViewer<T extends Object>{

	/**
	 * the list, and placement info
	 */
	public ArrayList<T> objectList;
	public int lowerBound = 0;
	public int selectedObject = -1; // default to -1 as checker for none selected

	/**
	 * the button info
	 */
	private int[][] buttonBounds = new int[2][4];
	private String[] buttonTexts = new String[2];
	
	/**
	 * the title
	 */
	private String title;
	
	/**
	 * sets up the new list viewer with the provided title and list of objects
	 * @param title
	 * @param objectList
	 */
	public UCViewer2DListViewer(String title, ArrayList<T> objectList){
		this.objectList = objectList;
		this.title = title;
	}
	
	/**
	 * setup the buttons
	 */
	public void setupButtons(int width, int height){
		buttonBounds[0][0] = 9 * (width / 10);
		buttonBounds[0][1] = 2 * (height / 20);
		buttonBounds[0][2] = 39 * (width / 40);
		buttonBounds[0][3] = 4 * (height / 20);
		buttonTexts[0] = "up";
		buttonBounds[1][0] = 9 * (width / 10);
		buttonBounds[1][1] = 14 * (height / 20);
		buttonBounds[1][2] = 39 * (width / 40);
		buttonBounds[1][3] = 16 * (height / 20);
		buttonTexts[1] = "down";
	}
	
	/**
	 * paint this component, list and buttons
	 * @param g
	 */
	public void paint(Graphics g, int width, int height){
		Font tempFont = new Font("Courier New", Font.BOLD, 40);
		g.setFont(tempFont);
		
		// paint list border 
		g.setColor(Color.black);
		g.drawRect( width / 10, height / 10, 8 * (width / 10), 7 * (height / 10));
		
		// paint title
		g.drawString(title, width / 10, height / 20);
		
		// fill list (highlight selected if any)
		int maxDisplayCount = 0;
		int tempHeightValue = 7 * (height / 10);
		while(tempHeightValue > 45){
			tempHeightValue -= 45;
			maxDisplayCount++;
		}
		int index = 0;
		for(T o : objectList){
			// if in bound
			if((index >= lowerBound)&&(index < (lowerBound + maxDisplayCount))){
				// check if selected
				if(selectedObject == index){
					g.setColor(Color.lightGray);
					g.fillRect(width / 10, (45 * (index - lowerBound)) + (height / 10), 8 * (width / 10), 45);
				}else{
					g.setColor(Color.black);
					g.drawRect(width / 10, (45 * (index - lowerBound)) + (height / 10), 8 * (width / 10), 45);
				}
				g.setColor(Color.black);
				g.drawString(o.toString(), 10 + (width / 10), (45 * (index - lowerBound)) + (height / 10) + 35);
			}
			
			index++; // increment
		}
		
		// paint buttons
		g.setColor(Color.black);
		Font butFont = g.getFont();
		butFont = butFont.deriveFont(Font.BOLD, 30);
		g.setFont(butFont);
		// cycle through each and paint
		for(int i = 0; i < buttonBounds.length; i++){
			int w =  buttonBounds[i][2] - buttonBounds[i][0];
			int h = buttonBounds[i][3] - buttonBounds[i][1];
			g.drawRect(buttonBounds[i][0], buttonBounds[i][1], w, h);
			
			g.drawString(buttonTexts[i], buttonBounds[i][0] + 5, buttonBounds[i][1] + ( 3 * (h / 4)));
		}
	}
	
	/**
	 * @return the name of the selected object
	 */
	public String getSelectedObjectName(){
		return objectList.get(selectedObject).toString();
	}
	
	/**
	 * checks if an object selection has been made
	 * @param p
	 * @param height
	 * @param width
	 * @return
	 */
	public int selectionMade(Point p, int height, int width){
		int selectionChoice = -1; // default
		
		// find selection
		if((p.x >= (height / 10))&&(p.x <= (9 * (width / 10)))){
			int y = height / 10;
			int i = lowerBound;
			while(y <= 7 * (height / 10)){
				if((p.y >= y)&&(p.y <= (y + 45))){
					if(i < objectList.size()){
						selectionChoice = i;
					}
				}
				y += 45; i++; // increment
			}
		}
		
		return selectionChoice; // return
	}

	/**
	 * checks which button is clicked, if any
	 * @param p
	 * @return the index of the button in the list, or -1 if none
	 */
	public int buttonClicked(Point p){
		int choice = -1; // default
		
		// find the button
		for(int i = 0; i < buttonBounds.length; i++){
			if((p.x >= buttonBounds[i][0])&&(p.x <= buttonBounds[i][2])){
				if((p.y >= buttonBounds[i][1])&&(p.y <= buttonBounds[i][3])){
					choice = i;
				}
			}
		}
		
		return choice; // return found, if any
	}
}
